@@ -1,21 +1,38 @@
 /*
  *   stunnel       Universal SSL tunnel
- *   Copyright (c) 1998-2006 Michal Trojnara <Michal.Trojnara@mirt.net>
- *                 All Rights Reserved
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
+ *   Copyright (C) 1998-2008 Michal Trojnara <Michal.Trojnara@mirt.net>
  *
+ *   This program is free software; you can redistribute it and/or modify it
+ *   under the terms of the GNU General Public License as published by the
+ *   Free Software Foundation; either version 2 of the License, or (at your
+ *   option) any later version.
+ * 
  *   This program is distributed in the hope that it will be useful,
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ *   See the GNU General Public License for more details.
+ * 
+ *   You should have received a copy of the GNU General Public License along
+ *   with this program; if not, see <http://www.gnu.org/licenses>.
+ * 
+ *   Linking stunnel statically or dynamically with other modules is making
+ *   a combined work based on stunnel. Thus, the terms and conditions of
+ *   the GNU General Public License cover the whole combination.
+ * 
+ *   In addition, as a special exception, the copyright holder of stunnel
+ *   gives you permission to combine stunnel with free software programs or
+ *   libraries that are released under the GNU LGPL and with code included
+ *   in the standard release of OpenSSL under the OpenSSL License (or
+ *   modified versions of such code, with unchanged license). You may copy
+ *   and distribute such a system following the terms of the GNU GPL for
+ *   stunnel and the licenses of the other code concerned.
+ * 
+ *   Note that people who make modified versions of stunnel are not obligated
+ *   to grant this special exception for their modified versions; it is their
+ *   choice whether to do so. The GNU General Public License gives permission
+ *   to release a modified version without this exception; this exception
+ *   also makes it possible to release a modified version which carries
+ *   forward this exception.
  */
 
 #ifndef PROTOTYPES_H
@@ -56,24 +73,23 @@ extern volatile int num_clients;
 
 void main_initialize(char *, char *);
 void main_execute(void);
+#if !defined (USE_WIN32) && !defined (__vms) && !defined(USE_OS2)
+void drop_privileges(void);
+#endif
 void stunnel_info(int);
+void die(int);
 
 /**************************************** Prototypes for log.c */
 
 void log_open(void);
 void log_close(void);
+void log_flush(void);
 void s_log(int, const char *, ...)
 #ifdef __GNUC__
     __attribute__ ((format (printf, 2, 3)));
 #else
     ;
 #endif
-void log_raw(const char *, ...)
-#ifdef __GNUC__
-    __attribute__ ((format (printf, 1, 2)));
-#else
-    ;
-#endif
 void ioerror(const char *);
 void sockerror(const char *);
 void log_error(int, int, const char *);
@@ -121,8 +137,7 @@ typedef struct {
 #endif
     unsigned long dpid;
     char *pidfile;
-    char *setuid_user;
-    char *setgid_group;
+    int uid, gid;
 #endif
 
         /* Win32 specific data for gui.c */
@@ -131,7 +146,7 @@ typedef struct {
 #endif
 
         /* logging-support data for log.c */
-    int debug_level;                               /* debug level for syslog */
+    int debug_level;                              /* debug level for logging */
 #ifndef USE_WIN32
     int facility;                               /* debug facility for syslog */
 #endif
@@ -139,11 +154,15 @@ typedef struct {
 
         /* on/off switches */
     struct {
-        unsigned int foreground:1;
-        unsigned int syslog:1;                              /* log to syslog */
         unsigned int rand_write:1;                    /* overwrite rand_file */
 #ifdef USE_WIN32
         unsigned int taskbar:1;                   /* enable the taskbar icon */
+#else /* !USE_WIN32 */
+        unsigned int foreground:1;
+        unsigned int syslog:1;
+#endif
+#ifdef USE_FIPS
+        unsigned int fips:1;                       /* enable FIPS 140-2 mode */
 #endif
     } option;
 } GLOBAL_OPTIONS;
@@ -160,6 +179,9 @@ typedef struct local_options {
     char *servname;        /* service name for logging & permission checking */
     SSL_SESSION *session;                           /* jecently used session */
     char local_address[IPLEN];             /* dotted-decimal address to bind */
+#ifndef USE_FORK
+    int stack_size;                            /* stack size for this thread */
+#endif
 
         /* service-specific data for ctx.c */
     char *ca_dir;                              /* directory for hashed certs */
@@ -176,6 +198,7 @@ typedef struct local_options {
 #if SSLEAY_VERSION_NUMBER >= 0x00907000L
     SOCKADDR_LIST ocsp_addr;
     char *ocsp_path;
+    unsigned long ocsp_flags;
 #endif /* OpenSSL-0.9.7 */
     SSL_METHOD *(*client_method)(void), *(*server_method)(void);
 
@@ -194,7 +217,9 @@ typedef struct local_options {
         /* protocol name for protocol.c */
     char *protocol;
     char *protocol_host;
-    char *protocol_credentials;
+    char *protocol_username;
+    char *protocol_password;
+    char *protocol_authentication;
 
         /* on/off switches */
     struct {
@@ -203,6 +228,7 @@ typedef struct local_options {
         unsigned int delayed_lookup:1;
         unsigned int accept:1;
         unsigned int remote:1;
+        unsigned int retry:1; /* loop remote+program */
 #ifndef USE_WIN32
         unsigned int program:1;
         unsigned int pty:1;
@@ -243,6 +269,10 @@ void parse_config(char *, char *);
 void context_init(LOCAL_OPTIONS *);
 void sslerror(char *);
 
+/**************************************** Prototypes for verify.c */
+
+void verify_init(LOCAL_OPTIONS *);
+
 /**************************************** Prototypes for network.c */
 
 #ifdef USE_POLL
@@ -284,7 +314,7 @@ typedef struct {
 
 typedef struct {
     LOCAL_OPTIONS *opt;
-    char accepting_address[IPLEN], connecting_address[IPLEN]; /* text */
+    char accepted_address[IPLEN]; /* text */
     SOCKADDR_LIST peer_addr; /* Peer address */
     FD local_rfd, local_wfd; /* Read and write local descriptors */
     FD remote_fd; /* Remote file descriptor */
@@ -309,7 +339,7 @@ extern int max_clients;
 extern int max_fds;
 #endif
 
-void *alloc_client_session(LOCAL_OPTIONS *, int, int);
+CLI *alloc_client_session(LOCAL_OPTIONS *, int, int);
 void *client(void *);
 int connect_wait(CLI *);
 
@@ -347,7 +377,7 @@ char *s_ntop(char *, SOCKADDR_UNION *);
 
 typedef enum {
     CRIT_KEYGEN, CRIT_INET, CRIT_CLIENTS, CRIT_WIN_LOG, CRIT_SESSION,
-    CRIT_SECTIONS
+    CRIT_LIBWRAP, CRIT_SSL, CRIT_SECTIONS
 } SECTION_CODE;
 
 void enter_critical_section(SECTION_CODE);
@@ -355,10 +385,10 @@ void leave_critical_section(SECTION_CODE);
 void sthreads_init(void);
 unsigned long stunnel_process_id(void);
 unsigned long stunnel_thread_id(void);
-int create_client(int, int, void *, void *(*)(void *));
+int create_client(int, int, CLI *, void *(*)(void *));
 #ifdef USE_UCONTEXT
 typedef struct CONTEXT_STRUCTURE {
-    char stack[STACK_SIZE];
+    char *stack; /* CPU stack for this thread */
     unsigned long id;
     ucontext_t ctx;
     s_poll_set *fds;
@@ -381,12 +411,12 @@ void stack_info(int);
 
 typedef struct {
     LOCAL_OPTIONS *section;
-    char pass[STRLEN];
+    char pass[PEM_BUFSIZE];
 } UI_DATA;
 
 #ifdef USE_WIN32
 void win_log(char *);
-void exit_stunnel(int);
+void exit_win32(int);
 int passwd_cb(char *, int, int, void *);
 #ifdef HAVE_OSSL_ENGINE_H
 int pin_cb(UI *, UI_STRING *);
@@ -428,6 +458,11 @@ LPTSTR str2tstr(const LPSTR);
 LPSTR tstr2str(const LPTSTR);
 #endif
 
+/**************************************** Prototypes for libwrap.c */
+
+void libwrap_init(int);
+void auth_libwrap(CLI *);
+
 #endif /* defined PROTOTYPES_H */
 
 /* End of prototypes.h */
